#include <c4d.h>
#include "messiahC4Dscene_events_handler.h"

// old config file code, temporary
#include "ConfigData.h"

// C/C++ Standard Libraries
#include <direct.h>

// Plugin Registration Functions   //////////////////////////////////////////////////////////////////////////

extern Bool RegisterMessiahDeformer(void);
extern Bool RegisterMessiahDeformTag(void);
extern Bool RegisterMessiahCommand(void);
extern Bool RegisterMessiahSceneHook(void);
extern Bool RegisterMessiahMover(void);

extern void messagebox( char const *msg );

// Globals  //////////////////////////////////////////////////////////////////////////

extern mh::module *	messiahHost;	// the messiahHOST module (an instance of messiah), only one for now

messiahC4Dscene_events_handler *scene_handler = NULL;

char progName[] = "Cinema4D 8.1";
char plugName[] = "messiahC4D";
char pluginDir[2048] = {0};

ConfigData *config = NULL;

bool messiah_loaded = false;

// Entry Point (PluginStart)  //////////////////////////////////////////////////////////////////////////

Bool PluginStart()
{
	Filename plugPath;

	// Initialize resources
	if( !resource.Init() ) return FALSE; 

	// Register the deformer plugin and its tag
	if( !RegisterMessiahSceneHook()		|| 
		!RegisterMessiahDeformTag()		||
		!RegisterMessiahCommand()		||
		!RegisterMessiahDeformer()		||
		!RegisterMessiahMover()			) return FALSE;

	// Retrieve, store and change to the plugin directory
	plugPath = GeGetPluginPath();
	String tmpstr;
	LONG len;
	tmpstr = plugPath.GetString();
	len= tmpstr.GetCStringLen() + 1;
	tmpstr.GetCString(pluginDir, len);
	
	// Load the config file
	//		This is old code from the last rev, will update this as time permits
	_chdir( pluginDir );
	config = ReadConfig();
	if( !config ) return FALSE;

	// Initialize the messiahHost module
	try
	{
		_chdir( config->messiahPath );
		mh::module::init();
		scene_handler = new messiahC4Dscene_events_handler;
		messiahHost->getScene()->setEventHandler( scene_handler );
		messiah_loaded = true;
	}
	catch( mh::exception &e )
	{
		messagebox( e.what() );
		return FALSE;			// shouldn't this cause the plugin to be unloaded?? We can't continue from here
	}

	// Inform the module of Cinema's capabilities
	mh::host_caps hcaps;

	hcaps.d_lwoCaps = mh::host_caps::e_lwoCaps::CanConstructGeom;
	messiahHost->setHostCaps( hcaps );


	return TRUE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// Exit Point (PluginEnd)  //////////////////////////////////////////////////////////////////////////

void PluginEnd()
{
	// Write the config file
	_chdir( pluginDir );
	WriteConfig();

	// Shutdown messiah
	if( messiahHost->isMessiahActive() )
		messiahHost->shutdown();
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// Message Handler (PluginMessage)  //////////////////////////////////////////////////////////////////////////

Bool PluginMessage(LONG id, void *data)
{
	switch( id )
	{
	case C4DMSG_PRIORITY:
		{
			SetPluginPriority( data, C4DPL_INIT_PRIORITY_PLUGINS );
			return TRUE;
		}
		break;
	}

	return TRUE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////
