// ============				MH_System.h				==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Class Name: MH_System
//
// ===============================================================
//
// Class Description:
//
// This class provides an interface to the messiahHOST system
// functions. The resulting object is responsible for the loading
// and freeing of the messiahHOST.* system object therefore
// the object is a Singleton to ensure that the library is loaded
// and freed only once. Access to the object is obtained through 
// the static 'inst' method. 
//
// The first call to inst() must pass the relavant information for
// system startup, see inst() definition for details.
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================
#ifndef MH_System_H
#define MH_System_H

#ifdef	__cplusplus

#define MESSIAH_HOST_MODULE

#include <string>
#include <cstring>
#include <cstdio>

typedef void *	(*MessiahGlobalFunc) (char[]);

class MH_System
{
public:

	virtual		~MH_System();

	double		CurrentFrame();
	void		CheckTime(double time, int flags);
	int			Activated();
	int			ShowInterface();

	// first call must pass the name of the host application i.e. LightWave and the name of
	// the connection module i.e. messiahLW.p
	// after initial call no arguments need to be passed.
	static MH_System *	Inst(char const *hostAppName = 0, 
							 char const *hostModuleName = 0,
							 void *(*hostGlobal)(char *) = 0);
	static void			DestroyInst();

	static double		scaleFactor;

protected:

	/*CST*/					MH_System(char const *hostAppName, char const *hostModuleName);
	/*CST*/					MH_System();

	virtual		bool		_loadMessiahHost()		{ return false; }
	virtual		bool		_unloadMessiahHost()	{ return false; }
	bool					_loadAllGlobals();
	void					_unloadAllGlobals();
	int						_beginSession();
	void					_endSession();

	std::string				_hostModuleName;
	std::string				_hostAppName;
	static		void *		(*_hostGlobal)(char *);

private:

	static	MH_System *	_instance;

};


typedef MH_System MHS;

#endif	// __cplusplus
#endif	// MH_System_H