// ============		messiahDeformerNode.h			==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Class Name: messiahDeformer
//
// ===============================================================
//
// Class Description:
//
// This class is a Maya Dependancy Graph Node.
//
// Attributes:
//
//		time:			input from time1.outTime to trigger a refresh
//		hostItem:		input from trans.msg to indicate the item to display in messiah
//		loadId:			value to store in the scene file, used for re-connection to messiah
//
// Associated Command:
//
//		messiah -attachDform
//
//			Call this command while the transform node is selected to attach it.
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================
#ifndef MM_MESSIAHDEFORMER
#define MM_MESSIAHDEFORMER

#include "ErrorCheck.h"
#include "Params.h"

#include "messiahCommonNodeOps.h"
#include <maya/MPxDeformerNode.h>

// ===============================================================
// messiahDeformer
// ===============================================================
class messiahDeformer : public MPxDeformerNode, public commonNode
{
public:
	
	/*Cst*/				messiahDeformer();
	/*Dst*/				~messiahDeformer();

	static void *		creator();
	static MStatus		initialize();

	MStatus				compute( MPlug const &plug, MDataBlock &dataBlock);
	MStatus				connectionMade(const MPlug &plug, const MPlug &otherPlug, bool asSrc);

	virtual MStatus		deform(	MDataBlock &		block,
								MItGeometry &		iter,
								MMatrix const &		mat,
								unsigned int		multiIndex);

	static MTypeId		id;

public:		// Attributes

	static MObject		hostItem;
	static MObject		loadId;
	static MObject		time;

};


#endif	// MM_MESSIAHDEFORMER