// ==========			messiahNode.cpp					==========
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Description: 
//
// Implementation of messiahNode
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/28/2002
//
// Revised By:
// Date: 
//
// ===============================================================

#include "messiahNode.h"
#include <maya/MFnTypedAttribute.h>
#include <maya/MGlobal.h>

// ===============================================================
// Initialize node type id
// ===============================================================
MTypeId     messiahNode::id( MM_MAN_ID );

// ===============================================================
// Initialize attribs to NULL
// ===============================================================
MObject		messiahNode::fxsBuffer;
MObject		messiahNode::scaleFactor;

// ===============================================================
// Constructor
// ===============================================================
messiahNode::messiahNode() 
{
	// if we're loading set the scene node (it contains the fxs buffer )
	// TODO: this will need to be adjusted to allow merging
	if( MAN::Inst()->SceneState() & MAN::SceneLoading )
		MAN::Inst()->SceneNode( this );
}

// ===============================================================
// Destructor
// ===============================================================
messiahNode::~messiahNode() 
{

}

// ===============================================================
// Creator
// ===============================================================
void* messiahNode::creator()
{
	return new messiahNode();
}

// ===============================================================
// Initialize
// ===============================================================
MStatus messiahNode::initialize()
{
	MFnTypedAttribute		tAttr, tAttr2;

	MStatus					stat;

	messiahNode::fxsBuffer = tAttr.create( MM_ATR_FXSBUFF_L, MM_ATR_FXSBUFF_S, MFnData::kString );
	tAttr.setReadable( true );
	tAttr.setWritable( true );
	tAttr.setHidden( true );
	tAttr.setKeyable( false );
	tAttr.setStorable( true );


	messiahNode::scaleFactor = tAttr2.create( MM_ATR_SCALE_L, MM_ATR_SCALE_S, MFnData::kString);
	tAttr2.setReadable( true );
	tAttr2.setWritable( true );
	tAttr2.setHidden( true );
	tAttr2.setKeyable( false );
	tAttr2.setStorable( true );
	
	stat = addAttribute( messiahNode::fxsBuffer );
		if (!stat) { stat.perror("addAttribute"); return stat;}
	stat = addAttribute( messiahNode::scaleFactor );
		if (!stat) { stat.perror("addAttribute"); return stat;}

	return MS::kSuccess;

}

// ===============================================================
// Should Save
// ===============================================================
MStatus	messiahNode::shouldSave(MPlug const &plug, bool &ret)
{
	if( plug == fxsBuffer )
	{
		ret = true;
		return MS::kSuccess;
	}

	return MS::kUnknownParameter;
}