// ============		messiahTransformerNode.h		==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Class Name: messiahTransformer
//
// ===============================================================
//
// Class Description:
//
// This class is a Maya Dependancy Graph Node.
//
// Attributes:
//
//		time:			input from time1.outTime to trigger a refresh
//		hostItem:		input from trans.msg to indicate the item to display in messiah
//		loadId:			value to store in the scene file, used for re-connection to messiah
//		translate:		values for output to the trans node's translation
//		rotate:			values for output to the trans node's rotation
//		rotateOrder:	static value that sets the rotate order to match messiah's
//
// Associated Command:
//
//		messiah -attachXform
//
//			Call this command while the transform node is selected to attach it.
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================
#ifndef MM_MESSIAHXFORMER
#define MM_MESSIAHXFORMER

#include "ErrorCheck.h"
#include "Params.h"
#include "messiahCommonNodeOps.h"

#include <maya/MPxNode.h>

// ===============================================================
// messiahTransformer
// ===============================================================
class messiahTransformer : public MPxNode, public commonNode
{
public:
	/*CST*/				messiahTransformer();
	/*DST*/				~messiahTransformer(); 

	static  void*		creator();
	static  MStatus		initialize();

	MStatus				connectionMade(const MPlug &plug, const MPlug &otherPlug, bool asSrc);
	virtual MStatus		compute( const MPlug& plug, MDataBlock& data );

	static	MTypeId		id;

public:		// Attributes

	static	MObject		hostItem;
	static	MObject		loadId;

	static	MObject		time;

	static	MObject		translateX;
	static	MObject		translateY;
	static	MObject		translateZ;

	static	MObject		rotateX;
	static	MObject		rotateY;
	static	MObject		rotateZ;

	static	MObject		rotateOrder;

};

#endif	// MM_MESSIAHXFORMER
