// ==================		pluginMain.cpp			==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Description: 
//
// initializePlugin()
//
//		registers the commands and nodes associated with the plugin
//
// uninitializePlugin()
//
//		removes resources allocated by initializeNode()
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================

#include "messiahTransformerNode.h"
#include "messiahDeformerNode.h"
#include "messiahNode.h"
#include "messiahCommonNodeOps.h"
#include "messiahCommand.h"
#include "Params.h"
#include "ErrorCheck.h"

#include <direct.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <maya/MFnPlugin.h>
#include <Maya/MString.h>
#include <maya/MGlobal.h>

#include <maya/MSceneMessage.h>		// for scene callback registration
#include <maya/MDGMessage.h>		// for DG callback registration

// scene callbacks
static	MCallbackId		SceneBeforeOpenID;
static	MCallbackId		SceneAfterOpenID;

static	MCallbackId		SceneBeforeSaveID;
static	MCallbackId		SceneAfterSaveID;

static	MCallbackId		SceneBeforeNewID;
static	MCallbackId		SceneAfterNewID;

// DG callbacks
static	MStatus			DGTimeChangedID;




// ===============================================================
// Initialize Plugin
// ===============================================================
MStatus initializePlugin( MObject obj )
{ 
	int stat=0;	
	char path[1024];
	char txt[1024]="";

	_getcwd(path, 1024);

	MStatus   status;
	MFnPlugin plugin( obj, MM_VENDOR, MM_PLUGIN_VERSION, MM_API_VERSION);

	// ===============================================================
	// Register messiahXform node
	// ===============================================================
	status = plugin.registerNode( MM_MOVE_NAME, messiahTransformer::id, messiahTransformer::creator,
								  messiahTransformer::initialize );
	MM_CheckError(status, "Failed to register node");
	MGlobal::displayInfo( MString(MM_MOVE_NAME) + MString(" loaded") );

	// ===============================================================
	// Register messiahDform node
	// ===============================================================
	status = plugin.registerNode( MM_DISP_NAME, messiahDeformer::id, messiahDeformer::creator,
								  messiahDeformer::initialize , MPxNode::kDeformerNode );
	MM_CheckError(status, "Failed to register node");
	MGlobal::displayInfo( MString(MM_DISP_NAME) + MString(" loaded") );

	// ===============================================================
	// Register Manager node
	// ===============================================================
	status = plugin.registerNode( MM_MAN_NAME, messiahNode::id, messiahNode::creator,
								  messiahNode::initialize);
	MM_CheckError(status, "Failed to register node");
	MGlobal::displayInfo( MString(MM_MAN_NAME) + MString(" loaded") );
	
	// ===============================================================
	// Register messiah command
	// ===============================================================
	status = plugin.registerCommand( MM_CMD_NAME, messiahCommand::creator);
	MM_CheckError(status, "Failed to register command");
	MGlobal::displayInfo( MString(MM_CMD_NAME) + MString(" command loaded") );

	// ===============================================================
	// Register Scene Callbacks
	// ===============================================================
	SceneBeforeOpenID	= MSceneMessage::addCallback(MSceneMessage::kBeforeOpen, MAN::OnLoadPre );	
	SceneAfterOpenID	= MSceneMessage::addCallback(MSceneMessage::kAfterOpen,  MAN::OnLoadPost );	
	SceneBeforeSaveID	= MSceneMessage::addCallback(MSceneMessage::kBeforeSave, MAN::OnSavePre );	
	SceneAfterSaveID	= MSceneMessage::addCallback(MSceneMessage::kAfterSave,  MAN::OnSavePost );	
	SceneBeforeNewID	= MSceneMessage::addCallback(MSceneMessage::kBeforeNew, MAN::OnClearPre );	
	SceneAfterNewID		= MSceneMessage::addCallback(MSceneMessage::kAfterNew,  MAN::OnClearPost );	

	// ===============================================================
	// Register DG Callbacks
	// ===============================================================
	MDGMessage::addTimeChangeCallback( MAN::OnTimeChange, NULL, &DGTimeChangedID );

	// ===============================================================
	// Create instance of messiahHOST
	// ===============================================================
	if( MH_System::Inst(HOST_APP_NAME, HOST_MODULE_NAME, common::hostGlobal ) )
		MGlobal::displayInfo( MString("messiah started") );


	//create the messiah menu
//	sprintf(txt,"%s\\%s",path,"messiah.mel");
//	MGlobal::sourceFile(txt);
	stat = MGlobal::executeCommand("source messiah");
	stat = MGlobal::executeCommand("pmgCreateMenu");

	return status;
}

// ===============================================================
// Un-Initialize Plugin
// ===============================================================
MStatus uninitializePlugin( MObject obj)
{
	int stat=0;	
	MStatus   status;
	MFnPlugin plugin( obj );

	// ===============================================================
	// De-Register messiahXform node
	// ===============================================================
	status = plugin.deregisterNode( messiahTransformer::id );
	MM_CheckError(status, "Failed to un-register node");
	
	// ===============================================================
	// De-Register messiahDform node
	// ===============================================================
	status = plugin.deregisterNode( messiahDeformer::id );
	MM_CheckError(status, "Failed to un-register node");

	// ===============================================================
	// De-Register messiahNode
	// ===============================================================
	status = plugin.deregisterNode( messiahNode::id );
	MM_CheckError(status, "Failed to un-register node");

	// ===============================================================
	// De-Register messiah command
	// ===============================================================
	status = plugin.deregisterCommand( MM_CMD_NAME );
	MM_CheckError(status, "Failed to un-register command");

	// ===============================================================
	// De-Register scene callbacks
	// ===============================================================
	MSceneMessage::removeCallback( SceneBeforeOpenID );
	MSceneMessage::removeCallback( SceneAfterOpenID );
	MSceneMessage::removeCallback( SceneBeforeSaveID );
	MSceneMessage::removeCallback( SceneAfterSaveID );
	MSceneMessage::removeCallback( SceneBeforeNewID );
	MSceneMessage::removeCallback( SceneAfterNewID );

	// ===============================================================
	// De-Register DG callbacks
	// ===============================================================
	MDGMessage::removeCallback( DGTimeChangedID );



	//Destroy the menu we created
	stat = MGlobal::executeCommand("pmgDestroyMenu");



	// ===============================================================
	// Destroy instance of messiahHOST
	// ===============================================================
	//MH_NodeLogFuncs funcs;
	MH_NodeLogFuncs::ClearAllElements(2);
	MH_System::DestroyInst();

	return status;
}


