#ifndef __C4DGUI_H
#define __C4DGUI_H

#ifndef __API_INTERN__
	#include "ge_math.h"
	#include "c4d_string.h"
	#include "gui.h"
	#define IDC_OK	   1
	#define IDC_CANCEL 2

	enum
	{
		FORMAT_REAL									= 'frea',
		FORMAT_LONG									= 'flng',
		FORMAT_PERCENT							= 'fpct',
		FORMAT_DEGREE								= 'fdgr',
		FORMAT_METER								= 'fmet',
		FORMAT_FRAMES								= 'ffrm',
		FORMAT_SECONDS							= 'fsec',
		FORMAT_SMPTE								= 'fsmp'
	};

#else
	#include "ge_math.h"
	#include "ge_string.h"
	#include "gui_defs.h"
	#include "gui_newdialog.h"
	#include "gui_coredefs.h"
	#include "api_defs.h"

#endif

class BaseBitmap;
class BaseContainer;
class GeResource;
class BitmapButton;
class Filename;
class SubDialog;

// for popupcreation
#define FIRST_POPUP_ID (900000)

#define BMPBUTTON_COPYBITMAP	1
#define BMPBUTTON_ACTION			2

#define IDM_CM_CLOSEWINDOW		12097

#define MAXTIME								10000      // maximum time value equal 2.77 hours!

class GeUserArea
{
	friend class GeDialog;

	private:
		Bool owncu;
		CUserArea *cu;

	protected:
		GeDialog  *dlg;

	public:

		GeUserArea(void);
		virtual ~GeUserArea(void);

		CUserArea *Get() { return cu; }
		void Set(CUserArea *t_cu) { cu = t_cu; owncu=TRUE; }

		// for overloading (for easy message handling)
		virtual Bool Init         (void);
		virtual Bool InitValues   (void);
		virtual Bool GetMinSize   (LONG &w,LONG &h);
		virtual void Sized        (LONG w,LONG h);
		virtual void Draw         (LONG x1,LONG y1,LONG x2,LONG y2);
		virtual Bool InputEvent   (const BaseContainer &msg);
		virtual Bool CoreMessage  (LONG id,const BaseContainer &msg);
		virtual void Timer        (const BaseContainer &msg);

		// for overloading (for special message handling)
		virtual LONG Message      (const BaseContainer &msg,BaseContainer &result);

		// functions to call
		void Redraw               (Bool threaded=FALSE);
		Bool SendParentMessage    (const BaseContainer &msg);

		LONG GetId                (void);
		LONG GetWidth             (void);
		LONG GetHeight            (void);
		Bool IsEnabled						(void);

		// input events and tTimer
		void SetTimer             (LONG timer);
		Bool GetInputState        (LONG askdevice,LONG askchannel,BaseContainer &res);
		Bool GetInputEvent        (LONG askdevice,BaseContainer &res);
		void KillEvents           (void);

		// pens
		void DrawSetPen           (const Vector &color);
		void DrawSetPen           (LONG id);
		void DrawSetTextCol       (LONG fg,LONG bg);
		void DrawSetTextCol       (const Vector &fg,LONG bg);
		void DrawSetTextCol       (LONG fg,const Vector &bg);
		void DrawSetTextCol       (const Vector &fg,const Vector &bg);

		// draw functions
		void DrawLine             (LONG x1,LONG y1,LONG x2,LONG y2);
		void DrawRectangle        (LONG x1,LONG y1,LONG x2,LONG y2);
		void DrawBitmap           (BaseBitmap *bmp,LONG wx,LONG wy,LONG ww,LONG wh,LONG x,LONG y,LONG w,LONG h,LONG mode);
		void DrawText             (const String &txt,LONG x,LONG y);
		void FillBitmapBackground (BaseBitmap *bmp,LONG offsetx,LONG offsety);

		// fonts
		void DrawSetFont          (LONG fontid);
		LONG DrawGetTextWidth     (const String &text);
		LONG DrawGetFontHeight    (void);

		// others
		void SetClippingRegion    (LONG x,LONG y,LONG w,LONG h);
		void ClearClippingRegion  (void);
		Bool OffScreenOn          (void);
		void ScrollArea           (LONG xdiff,LONG ydiff,LONG x,LONG y,LONG w,LONG h);
		Bool Global2Local         (LONG *x,LONG *y);

		void LayoutChanged        (void);

		Bool GetDragPosition      (const BaseContainer &msg,LONG *x,LONG *y);
		Bool GetDragObject        (const BaseContainer &msg,LONG *type,void **object);
		Bool HandleMouseDrag      (const BaseContainer &msg,LONG type,void *data,Bool privateframe);
		Bool SetDragDestination   (LONG cursor);
		void GetBorderSize        (LONG type,LONG *l,LONG *t,LONG *r,LONG *b);
		void DrawBorder           (LONG type,LONG x1,LONG y1,LONG x2,LONG y2);

#ifdef __API_INTERN__
		Bool OpenPopUpMenu				(LONG menuid,LONG localx,LONG localy);
#endif
};


template <class TYPE> class TriState
{
		LONG tri;
		TYPE value;

	public:
		TriState()							{ tri = -1; value = TYPE(); }
		Bool GetTri() const			{ return tri==1; }
		TYPE GetValue() const		{ return value;  }
		Bool HasChanged() const { return tri!=-1; }
		void Add(TYPE val)
		{
			if (tri==-1)
			{
				tri = 0;
				value = val;
			}
			else if (tri==0 && value!=val)
			{
				tri = 1;
			}
		}
		void Add(const BaseContainer &bc,LONG id);
		LONG Check(GeDialog *dlg,const BaseContainer &msg,LONG cid,LONG gid);
		LONG CheckVector(GeDialog *dlg,const BaseContainer &msg,LONG current_id,LONG gidx,LONG gidy,LONG gidz);
		LONG CheckBaseTime(GeDialog *dlg,const BaseContainer &msg,BaseDocument *doc,LONG cid,LONG gid);
};


class GeDialog
{
	friend class GeUserArea;
	friend class _GeListView;

	private:
		CDialog				*cd;
		BitmapButton	*bb_first;
	protected:
		Bool					createlayout;

	public:
		GeDialog(void);
		virtual ~GeDialog(void);

		CDialog *Get() { return cd; }

		// for overloading (for easy message handling)
		virtual Bool CreateLayout (void);
		virtual Bool InitValues   (void);
		virtual Bool CoreMessage  (LONG id,const BaseContainer &msg);
		virtual Bool Command      (LONG id,const BaseContainer &msg);
		virtual Bool AskClose     (void);
		virtual void Timer        (const BaseContainer &msg);

		// for overloading (for special message handling)
		virtual LONG Message      (const BaseContainer &msg,BaseContainer &result);

		// functions to call
		Bool Open                 (Bool async,LONG pluginid,LONG xpos=-1,LONG ypos=-1);
		Bool Close                (void);
		Bool SendMessage          (LONG id,const BaseContainer &msg);
		void SetTimer             (LONG timer);
		void SetTitle							(const String &title);

		// set/get functions
		Bool Enable               (LONG id, Bool enabled);
		Bool SetBool              (LONG id, Bool value,LONG tristate=0);
		Bool SetLong              (LONG id, LONG value,LONG min=-MAXLONG,LONG max=MAXLONG,LONG step=1,LONG tristate=0);
		Bool SetReal              (LONG id, Real value,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0,LONG format=FORMAT_REAL,Real min2=0.0,Real max2=0.0,Bool quadscale=FALSE,LONG tristate=0);
		Bool SetMeter             (LONG id, Real value,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0,LONG tristate=0);
		Bool SetDegree            (LONG id, Real value,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0,LONG tristate=0);
		Bool SetPercent           (LONG id, Real value,Real min=0.0,Real max=100.0,Real step=1.0,LONG tristate=0);
		Bool SetTime							(LONG id, BaseDocument *doc, const BaseTime &value,const BaseTime &min=BaseTime(-MAXTIME,1), const BaseTime &max=BaseTime(MAXTIME,1),LONG stepframes = 1,LONG tristate=0);
		Bool SetVectorReal        (LONG id_x, LONG id_y, LONG id_z, const Vector &value, Real min=-MAXREAL, Real max=MAXREAL, Real step=1.0,LONG tristate=0);
		Bool SetVectorMeter       (LONG id_x, LONG id_y, LONG id_z, const Vector &value, Real min=-MAXREAL, Real max=MAXREAL, Real step=1.0,LONG tristate=0);
		Bool SetVectorDegree      (LONG id_x, LONG id_y, LONG id_z, const Vector &value, Real min=-MAXREAL, Real max=MAXREAL, Real step=1.0,LONG tristate=0);
		Bool SetVectorPercent     (LONG id_x, LONG id_y, LONG id_z, const Vector &value, Real min=0.0, Real max=100.0, Real step=1.0,LONG tristate=0);
		Bool SetString            (LONG id, const String &text,LONG tristate=0);
		Bool SetColorField        (LONG id, const Vector &color, Real brightness, Real maxbrightness, LONG flags);
		Bool SetFilename          (LONG id, const Filename &fn,LONG tristate=0);

		Bool GetBool              (LONG id, Bool &value);
		Bool GetLong              (LONG id, LONG &value);
		Bool GetReal              (LONG id, Real &value);
		Bool GetVector            (LONG id_x, LONG id_y, LONG id_z, Vector &value);
		Bool GetString            (LONG id, String &text);
		Bool GetColorField        (LONG id, Vector &color, Real &brightness);
		Bool GetTime							(LONG id, BaseDocument *doc, BaseTime &time);
		Bool GetFilename          (LONG id, Filename &fn);

		Bool CheckTristateChange	(LONG id); // return wheter the gadget content has been changed since the last SetXXX or not

		// container set/get functions
		Bool SetBool              (LONG id, BaseContainer *bc, LONG bcid);
		Bool SetLong              (LONG id, BaseContainer *bc, LONG bcid,LONG min=-MAXLONG,LONG max=MAXLONG,LONG step=1);
		Bool SetReal              (LONG id, BaseContainer *bc, LONG bcid,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0,LONG format=FORMAT_REAL,Real min2=0.0,Real max2=0.0,Bool quadscale=FALSE);
		Bool SetMeter             (LONG id, BaseContainer *bc, LONG bcid,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0);
		Bool SetDegree            (LONG id, BaseContainer *bc, LONG bcid,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0);
		Bool SetPercent           (LONG id, BaseContainer *bc, LONG bcid,Real min=0.0,Real max=100.0,Real step=1.0);
		Bool SetTime							(LONG id, BaseDocument *doc,BaseContainer *bc, LONG bcid, const BaseTime &min=BaseTime(-MAXTIME,1), const BaseTime &max=BaseTime(MAXTIME,1),LONG stepframes=1);

		Bool SetVectorReal        (LONG id_x, LONG id_y, LONG id_z, BaseContainer *bc, LONG bcid, Real min=-MAXREAL, Real max=MAXREAL,Real step=1.0);
		Bool SetVectorMeter       (LONG id_x, LONG id_y, LONG id_z, BaseContainer *bc, LONG bcid, Real min=-MAXREAL, Real max=MAXREAL,Real step=1.0);
		Bool SetVectorDegree      (LONG id_x, LONG id_y, LONG id_z, BaseContainer *bc, LONG bcid, Real min=-MAXREAL, Real max=MAXREAL,Real step=1.0);
		Bool SetVectorPercent     (LONG id_x, LONG id_y, LONG id_z, BaseContainer *bc, LONG bcid, Real min=0.0, Real max=100.0,Real step=1.0);

		Bool SetString            (LONG id, BaseContainer *bc, LONG bcid);
		Bool SetFilename          (LONG id, BaseContainer *bc, LONG bcid);
		Bool SetColorField        (LONG id, BaseContainer *bc, LONG bc_colid, LONG bc_brightnessid, Real maxbrightness, LONG flags);
		Bool SetImage							(LONG id, BaseBitmap *bmp);
		Bool SetImage							(LONG id, const Filename &name);

		Bool GetBool              (LONG id, BaseContainer *bc, LONG bcid);
		Bool GetLong              (LONG id, BaseContainer *bc, LONG bcid);
		Bool GetReal              (LONG id, BaseContainer *bc, LONG bcid);
		Bool GetTime							(LONG id, BaseDocument *doc, BaseContainer *bc, LONG bcid);
		Bool GetVector            (LONG id_x, LONG id_y, LONG id_z, BaseContainer *bc, LONG bcid);
		Bool GetString            (LONG id, BaseContainer *bc, LONG bcid);
		Bool GetFilename          (LONG id, BaseContainer *bc, LONG bcid);
		Bool GetColorField        (LONG id, BaseContainer *bc, LONG bc_colid, LONG bc_brightnessid);

		// set/get functions for tristates
		Bool SetBool              (LONG id, const TriState<Bool> &tri);
		Bool SetLong              (LONG id, const TriState<LONG> &tri,LONG min=-MAXLONG,LONG max=MAXLONG,LONG step=1);
		Bool SetReal              (LONG id, const TriState<Real> &tri,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0,LONG format=FORMAT_REAL,Real min2=0.0,Real max2=0.0,Bool quadscale=FALSE);
		Bool SetMeter             (LONG id, const TriState<Real> &tri,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0);
		Bool SetDegree            (LONG id, const TriState<Real> &tri,Real min=-MAXREAL,Real max=MAXREAL,Real step=1.0);
		Bool SetPercent           (LONG id, const TriState<Real> &tri,Real min=0.0,Real max=100.0,Real step=1.0);
		Bool SetTime							(LONG id, BaseDocument *doc, const TriState<BaseTime> &tri,const BaseTime &min=BaseTime(-MAXTIME,1), const BaseTime &max=BaseTime(MAXTIME,1),LONG stepframes = 1);
		Bool SetVectorReal        (LONG id_x, LONG id_y, LONG id_z, const TriState<Vector> &tri, Real min=-MAXREAL, Real max=MAXREAL, Real step=1.0);
		Bool SetVectorMeter       (LONG id_x, LONG id_y, LONG id_z, const TriState<Vector> &tri, Real min=-MAXREAL, Real max=MAXREAL, Real step=1.0);
		Bool SetVectorDegree      (LONG id_x, LONG id_y, LONG id_z, const TriState<Vector> &tri, Real min=-MAXREAL, Real max=MAXREAL, Real step=1.0);
		Bool SetVectorPercent     (LONG id_x, LONG id_y, LONG id_z, const TriState<Vector> &tri, Real min=0.0, Real max=100.0, Real step=1.0);
		Bool SetString            (LONG id, const TriState<String> &tri);

		Bool CheckValueRanges     (void);

		Bool ScreenToLocal				(LONG *x, LONG *y);
		Bool LocalToScreen				(LONG *x, LONG *y);

		// layout stuff
		Bool LoadDialogResource   (LONG id, GeResource *lr, LONG flags);

		Bool AddCheckbox          (LONG id, LONG flags, LONG initw, LONG inith, const String &name);
		Bool AddButton            (LONG id, LONG flags, LONG initw, LONG inith, const String &name);
		Bool AddStaticText        (LONG id, LONG flags, LONG initw, LONG inith, const String &name,LONG borderstyle);
		Bool AddArrowButton       (LONG id, LONG flags, LONG initw, LONG inith, LONG arrowtype);
		Bool AddEditText          (LONG id, LONG flags, LONG initw=0, LONG inith=0);
		Bool AddMultiLineEditText (LONG id, LONG flags, LONG initw=0, LONG inith=0,LONG style=0);
		Bool AddEditNumber        (LONG id, LONG flags, LONG initw=80, LONG inith=0);
		Bool AddEditNumberArrows  (LONG id, LONG flags, LONG initw=70, LONG inith=0);
		Bool AddEditSlider        (LONG id, LONG flags, LONG initw=80, LONG inith=0);
		Bool AddSlider            (LONG id, LONG flags, LONG initw=90, LONG inith=0);
		Bool AddColorField        (LONG id, LONG flags, LONG initw=0,  LONG inith=0);
		Bool AddColorChooser      (LONG id, LONG flags, LONG initw=0,  LONG inith=0,LONG layoutflags=0);
		Bool AddRadioButton				(LONG id, LONG flags, LONG initw, LONG inith, const String &name);
		Bool AddRadioText					(LONG id, LONG flags, LONG initw, LONG inith, const String &name);

		Bool AddSeparatorH        (LONG initw);
		Bool AddSeparatorV        (LONG inith);

		Bool AddRadioGroup        (LONG id, LONG flags, LONG columns=1, LONG rows=0);
		Bool AddComboBox          (LONG id, LONG flags, LONG initw=0, LONG inith=0);
		Bool AddPopupButton       (LONG id, LONG flags, LONG initw=0, LONG inith=0);
		Bool AddListView          (LONG id, LONG flags, LONG initw=0, LONG inith=0);

		Bool AddChild             (LONG id, LONG subid,const String &child);
		Bool FreeChilds           (LONG id);
		Bool SetPopup							(LONG id,const BaseContainer &bc);

		Bool AddUserArea          (LONG id, LONG flags, LONG initw=0, LONG inith=0);
		Bool AttachUserArea       (GeUserArea &ua,LONG id);

		Bool AddSubDialog					(LONG id,LONG flags,LONG initw=0,LONG inith=0);
		Bool AttachSubDialog	    (SubDialog *userdlg,LONG id);

		Bool AddDlgGroup          (LONG type);
		Bool AttachImage					(LONG id, BaseBitmap *bmp, LONG flags, LONG border=BORDER_NONE);
		Bool AttachImage					(LONG id, const Filename &name, LONG flags, LONG border=BORDER_NONE);
		GeUserArea *GetUserArea		(LONG id);

		Bool TabGroupBegin        (LONG id, LONG flags,LONG tabtype=TAB_TABS);
		Bool GroupBegin           (LONG id, LONG flags,LONG cols,LONG rows,const String &title,LONG groupflags);
		Bool GroupEnd             (void);
		Bool GroupSpace           (LONG spacex,LONG spacey);
		Bool GroupBorder          (LONG borderstyle);
		Bool GroupBorderNoTitle		(LONG borderstyle);
		Bool GroupBorderSpace     (LONG left, LONG top,LONG right,LONG bottom);

		Bool GetItemDim           (LONG id,LONG *x,LONG *y,LONG *w,LONG *h);
		Bool GetDragPosition      (const BaseContainer &msg,LONG *x,LONG *y);
		Bool GetDragObject        (const BaseContainer &msg,LONG *type,void **object);
		Bool SetDragDestination   (LONG cursor);

		Bool GetVisibleArea       (LONG scrollgroupid,LONG *x1,LONG *y1,LONG *x2,LONG *y2);
		Bool SetVisibleArea       (LONG scrollgroupid,LONG x1,LONG y1,LONG x2,LONG y2);
		Bool ScrollGroupBegin     (LONG id, LONG flags,LONG scrollflags);

		Bool LayoutChanged        (LONG id);
		Bool Activate             (LONG id);

		Bool LayoutFlushGroup     (LONG id);

		Bool RestoreLayout				(LONG pluginid,void *secret);

		Bool MenuFlushAll         (void); // call to create a complete new menu
		Bool MenuSubBegin         (const String &string); // create a new menugroup
		Bool MenuSubEnd           (void); // close a menugroup
		Bool MenuAddCommand       (LONG cmdid); // add a command to the menugroup
		Bool MenuAddString        (LONG id,const String &string); // add a string to the menugroup
		Bool MenuAddSeparator     (void); // add a separator
		Bool MenuFinished         (void); // call when finished the menulayout
		Bool MenuInitString       (LONG id,Bool enabled,Bool checked);

		Bool CheckClose						(void);

#ifdef __API_INTERN__
		Bool MenuSetResource      (LONG id);
		Bool OpenPopUpMenu				(LONG menuid,LONG localx,LONG localy);
#endif
};

class GeModalDialog : public GeDialog
{	
	private:
		Bool dlg_result;
		Bool close;
	public:
		GeModalDialog(void);
		virtual ~GeModalDialog(void);

		Bool Open(LONG xpos=-1,LONG ypos=-1);
		Bool Close(Bool result);
		Bool GetResult(void) { return dlg_result; }

		// for overloading (for special message handling)
		virtual LONG Message(const BaseContainer &msg,BaseContainer &result);

		void CheckLong(LONG id, LONG type, LONG val, LONG x, LONG y);
		void CheckReal(LONG id, LONG type, Real val, Real x, Real y, LONG format=IS_NOTHING);
		void CheckMeter(LONG id, LONG type, Real val, Real x, Real y);
		void CheckPercent(LONG id, LONG type, Real val, Real x, Real y);
		void CheckDegree(LONG id, LONG type, Real val, Real x, Real y);
};

class SubDialog : public GeDialog
{
		Bool dlg_result;

	public:
		SubDialog(void);
		virtual ~SubDialog(void);
		virtual LONG Message(const BaseContainer &msg,BaseContainer &result);
};

inline LONG SizePixChr(LONG pixels,LONG chars) { return (pixels<<14)+chars; }
inline LONG SizePix(LONG pixels)               { return SizePixChr(pixels,0); }
inline LONG SizeChr(LONG chars)                { return SizePixChr(0,chars); }

#ifndef __API_INTERN__
	void MessageDialog(const String &str);
	void MessageDialog(LONG id);
	void MessageDialog(LONG id, const String &p1);
	void MessageDialog(LONG id, const String &p1, const String &p2);
	Bool QuestionDialog(const String &str);
	Bool QuestionDialog(LONG id);
	Bool QuestionDialog(LONG id, const String &p1);
	Bool QuestionDialog(LONG id, const String &p1, const String &p2);
	Bool GetInputEvent(LONG askdevice,BaseContainer &res);
	Bool GetInputState(LONG askdevice,LONG askchannel,BaseContainer &res);
#endif

#define MOUSEPOS (0x7FFFFFFF) // use this value for screenx/screeny
LONG ShowPopupMenu(CDialog *cd,LONG screenx,LONG screeny,const BaseContainer &bc,LONG flags=POPUP_RIGHT|POPUP_EXECUTECOMMANDS);

#endif
