#include "c4d_memory.h"
#include "c4d_basecontainer.h"
#include "c4d_file.h"

BaseContainer::BaseContainer(void)
{
	weak_link=FALSE;
	bc=C4DOS.Bc->Alloc(0);
	if (!bc) { bc=C4DOS.Bc->Default; weak_link=TRUE; }
}

BaseContainer::BaseContainer(const _BaseContainer *b, Bool t_weak_link)
{
	weak_link=t_weak_link;
	if (weak_link)
		bc=(_BaseContainer*)b;
	else
		bc=C4DOS.Bc->GetClone((_BaseContainer*)b);				  

	if (!bc) { bc=C4DOS.Bc->Default; weak_link=TRUE; }
}

BaseContainer::BaseContainer(LONG id)
{
	weak_link=FALSE;
	bc=C4DOS.Bc->Alloc(id);
	if (!bc) { bc=C4DOS.Bc->Default; weak_link=TRUE; }
}

BaseContainer::BaseContainer(const BaseContainer &n)
{
	if (!n.bc) return;
	if (n.weak_link)
	{
		weak_link=TRUE;	
		bc=n.bc; 
	}
	else
	{
		weak_link=FALSE;
		bc=C4DOS.Bc->GetClone(n.bc);
		if (!bc) { bc=C4DOS.Bc->Default; weak_link=TRUE; }
	}
} 

BaseContainer::~BaseContainer(void)
{
	if (!bc || weak_link) return; 
	C4DOS.Bc->Free(bc);
	bc=NULL;
}

const BaseContainer& BaseContainer::operator = (const BaseContainer& n)
{
	if (n.weak_link)
	{
		if (!weak_link) C4DOS.Bc->Free(bc);
		weak_link=TRUE;	
		bc=n.bc; 
	}
	else if (weak_link)
	{
		weak_link=FALSE;
		bc=C4DOS.Bc->GetClone(n.bc);
		if (!bc) { bc=C4DOS.Bc->Default; weak_link=TRUE; }
	}
	else
		C4DOS.Bc->CopyTo(n.bc,bc); 

	return *this;
}

BaseContainer BaseContainer::GetClone(void) const
{
	BaseContainer cc;
	if (bc && cc.bc)	
		C4DOS.Bc->CopyTo(bc,cc.bc); 
	return cc;
}

BaseContainer BaseContainer::GetInstance(void) const
{
	return BaseContainer(bc,TRUE);
}

Bool BaseContainer::operator == (const BaseContainer &c) const
{
	if (!bc || !c.bc) return FALSE;
	return C4DOS.Bc->Compare(bc,c.bc) && C4DOS.Bc->GetId(bc)==C4DOS.Bc->GetId(c.bc);
}

Bool BaseContainer::operator != (const BaseContainer &c) const
{
	if (!bc || !c.bc) return TRUE;
	return !(C4DOS.Bc->Compare(bc,c.bc) && C4DOS.Bc->GetId(bc)==C4DOS.Bc->GetId(c.bc));
}

void BaseContainer::FlushAll(void)
{
	if (!bc) return;
	C4DOS.Bc->FlushAll(bc);
} 

LONG BaseContainer::GetId(void) const
{
	if (!bc) return 0;
	return C4DOS.Bc->GetId(bc);
}

void BaseContainer::SetId(LONG c_id)
{
	if (!bc) return;
	C4DOS.Bc->SetId(bc,c_id);
}  

Bool BaseContainer::RemoveData(LONG id)
{
	if (!bc) return FALSE;
	return C4DOS.Bc->RemoveData(bc,id);
}

Bool BaseContainer::RemoveIndex(LONG i)
{
	if (!bc) return FALSE;
	return C4DOS.Bc->RemoveIndex(bc,i);
}

LONG BaseContainer::FindIndex(LONG id) const
{
	if (!bc) return -1;
	return C4DOS.Bc->FindIndex(bc,id);
}

LONG BaseContainer::GetIndexId(LONG index) const
{
	if (!bc) return -1;
	return C4DOS.Bc->GetIndexId(bc,index);
}

LONG BaseContainer::GetType(LONG id) const
{
	if (!bc) return BCTYPE_NONE;
	LONG ind=C4DOS.Bc->FindIndex(bc,id);
	if (ind==-1) return BCTYPE_NONE;
	return C4DOS.Bc->GetIndexType(bc,ind);
}

LONG BaseContainer::GetIndexType(LONG index) const
{
	if (!bc) return BCTYPE_NONE;
	return C4DOS.Bc->GetIndexType(bc,index);
}

Bool BaseContainer::GetBool(LONG id, Bool preset) const
{
	if (!bc || FindIndex(id)==-1) return preset;
	return C4DOS.Bc->GetBool(bc,id);
}

LONG BaseContainer::GetLong(LONG id, LONG preset) const
{
	if (!bc || FindIndex(id)==-1) return preset;
	return C4DOS.Bc->GetLong(bc,id);
}

Real BaseContainer::GetReal(LONG id, Real preset) const
{
	if (!bc || FindIndex(id)==-1) return preset;
	return C4DOS.Bc->GetReal(bc,id);
}

Vector BaseContainer::GetVector(LONG id, const Vector &preset) const
{																							 
	if (!bc || FindIndex(id)==-1) return preset;
	return C4DOS.Bc->GetVector(bc,id);
}

Matrix BaseContainer::GetMatrix(LONG id, const Matrix &preset) const
{
	if (!bc || FindIndex(id)==-1) return preset;
	return C4DOS.Bc->GetMatrix(bc,id);
}

String BaseContainer::GetString(LONG id, const String &preset) const
{
	if (!bc || FindIndex(id)==-1) return preset;
	return String(C4DOS.Bc->GetString(bc,id),TRUE);
}

Filename BaseContainer::GetFilename(LONG id, const Filename &preset) const
{
	if (!bc || FindIndex(id)==-1) return preset;
	return Filename(C4DOS.Bc->GetFilename(bc,id),TRUE);
}

BaseContainer BaseContainer::GetContainer(LONG id) const
{
	if (!bc || FindIndex(id)==-1) return BaseContainer();
	return BaseContainer(C4DOS.Bc->GetContainer(bc,id),FALSE);
}

BaseContainer BaseContainer::GetContainerInstance(LONG id) const
{
	if (!bc || FindIndex(id)==-1) return BaseContainer();
	_BaseContainer *x=C4DOS.Bc->GetContainer(bc,id);
	if (x) return BaseContainer(x,TRUE);
	return BaseContainer();
}

BaseTime BaseContainer::GetTime(LONG id, const BaseTime &preset) const
{
	if (!bc || FindIndex(id)==-1) return preset;
	return C4DOS.Bc->GetBaseTime(bc,id);
}

void BaseContainer::SetBool(LONG id, Bool b)
{
	if (!bc) return;
	C4DOS.Bc->SetBool(bc,id,b);
}

void BaseContainer::SetLong(LONG id, LONG l)
{
	if (!bc) return;
	C4DOS.Bc->SetLong(bc,id,l);
}

void BaseContainer::SetReal(LONG id, Real r)
{
	if (!bc) return;
	C4DOS.Bc->SetReal(bc,id,r);
}

void BaseContainer::SetVector(LONG id, const Vector &v)
{
	if (!bc) return;
	C4DOS.Bc->SetVector(bc,id,v);
}

void BaseContainer::SetMatrix(LONG id, const Matrix &v)
{
	if (!bc) return;
	C4DOS.Bc->SetMatrix(bc,id,v);
}

void BaseContainer::SetString(LONG id, const String &s)
{
	if (!bc || !s.Get()) return;
	C4DOS.Bc->SetString(bc,id,s.Get());
}

void BaseContainer::SetFilename(LONG id, const Filename &s)
{
	if (!bc || !s.Get()) return;
	C4DOS.Bc->SetFilename(bc,id,s.Get());
}

void BaseContainer::SetContainer(LONG id, const BaseContainer &s)
{
	if (!bc || !s.Get()) return;
	C4DOS.Bc->SetContainer(bc,id,s.Get());
}

void BaseContainer::SetTime(LONG id, const BaseTime &t)
{
	if (!bc) return;
	C4DOS.Bc->SetBaseTime(bc,id,t);
}

GeData *BaseContainer::GetIndexData(LONG index)
{
	if (!bc) return NULL;
	return C4DOS.Bc->GetIndexData(bc,index);
}

Bool BaseContainer::InsData(LONG id, GeData *n)
{
	if (!bc) return FALSE;
	return C4DOS.Bc->InsData(bc,id,n);
}

Bool BaseContainer::SetData(LONG id, GeData *n)
{
	if (!bc) return FALSE;
	return C4DOS.Bc->SetData(bc,id,n);
}

