#ifndef __C4DBASECONTAINER_H
#define __C4DBASECONTAINER_H

#include "ge_math.h"
#include "ge_matrix.h"
#include "c4d_basetime.h"
#include "c4d_string.h"
#include "c4d_file.h"

class _BaseContainer;

#define BCTYPE_NONE					0
#define BCTYPE_LONG				 15
#define BCTYPE_REAL				 19
#define BCTYPE_TIME				 22
#define BCTYPE_VECTOR			 23
#define BCTYPE_MATRIX			 25
#define BCTYPE_BYTEARRAY	128
#define BCTYPE_STRING			130
#define BCTYPE_FILENAME		131
#define BCTYPE_CONTAINER	132

class BaseContainer
{
	friend class BaseObject;

	private:
		_BaseContainer *bc;
		Bool weak_link;
	public:
		BaseContainer(void);
		BaseContainer(LONG id);              
		BaseContainer(const BaseContainer &n); 
		BaseContainer(const _BaseContainer *bc, Bool weak_link=FALSE);
		~BaseContainer(void);

		const BaseContainer &operator = (const BaseContainer &n);
		BaseContainer GetClone(void) const;
		BaseContainer GetInstance(void) const;

		void FlushAll(void); 

		LONG GetId(void) const; 
		void SetId(LONG c_id);  

		_BaseContainer *Get(void) const { return bc; }
		Bool RemoveData(LONG id);
		Bool RemoveIndex(LONG i);

		LONG FindIndex(LONG id) const;			
		LONG GetIndexId(LONG index) const;		

		LONG GetType(LONG id) const;
		LONG GetIndexType(LONG index) const;

		Bool operator == (const BaseContainer &d) const;
		Bool operator != (const BaseContainer &d) const;

		Bool 		 GetBool		(LONG id, Bool preset=FALSE) const;
		LONG		 GetLong    (LONG id, LONG preset=0) const;

		Real		 GetReal    (LONG id, Real preset=0.0) const;

		Vector	 GetVector  (LONG id, const Vector &preset=Vector()) const;
		Matrix	 GetMatrix  (LONG id, const Matrix &preset=Matrix()) const;

		String   GetString  (LONG id, const String   &preset=String()) const;
		Filename GetFilename(LONG id, const Filename &preset=Filename()) const;

		BaseTime GetTime    (LONG id, const BaseTime &preset=BaseTime()) const;

		BaseContainer GetContainer(LONG id) const;
		BaseContainer GetContainerInstance(LONG id) const;

		void SetBool    (LONG id, Bool b);
		void SetLong    (LONG id, LONG l);
		void SetReal    (LONG id, Real r);

		void SetVector  (LONG id, const Vector &v);
		void SetMatrix  (LONG id, const Matrix &m);

		void SetString  (LONG id, const String &s);
		void SetFilename(LONG id, const Filename &f);
		void SetTime    (LONG id, const BaseTime &b);

		void SetContainer(LONG id, const BaseContainer &s);

		GeData *GetIndexData(LONG index);
		Bool InsData(LONG id, GeData *n);
		Bool SetData(LONG id, GeData *n);
};

#endif
