#ifndef __C4DBASEDOCUMENT_H
#define __C4DBASEDOCUMENT_H

#include "c4d_baselist.h"
#include "c4d_file.h"
#include "c4d_tools.h"
#include "operatingsystem.h"

class BaseTag;
class TextureTag;
class BaseObject;
class BaseMaterial;
class Thread;

class RenderData : public BaseList2D
{
	private:
		RenderData(void);
		~RenderData(void);
	public:
		RenderData *next(void) { return (RenderData*)C4DOS.Bl->Next(this); }
		RenderData *pred(void) { return (RenderData*)C4DOS.Bl->Pred(this); }

		BaseContainer GetData(void);
		void SetData(const BaseContainer &bc);
};

class BaseDocument : public BaseList2D
{
	private:
		BaseDocument(void);
		~BaseDocument(void);
	public:
		BaseDocument *next(void) { return (BaseDocument*)C4DOS.Bl->Next(this); }
		BaseDocument *pred(void) { return (BaseDocument*)C4DOS.Bl->Pred(this); }

		BaseContainer GetData(LONG type);
		void SetData(LONG type, const BaseContainer &bc);

		BaseObject		*GetFirstObject(void);
		BaseMaterial	*GetFirstMaterial(void);
		RenderData		*GetFirstRenderData(void);

		void InsertRenderData(RenderData *rd, RenderData *pred=NULL);
		void InsertMaterial(BaseMaterial *mat, BaseMaterial *pred=NULL, Bool checknames=TRUE);
		void InsertObject(BaseObject *op, BaseObject *parent, BaseObject *pred, Bool checknames=TRUE);

		RenderData		*GetActiveRenderData(void);
		BaseObject		*GetActiveObject  (void);
		BaseMaterial	*GetActiveMaterial(void);
		TextureTag		*GetActiveTexTag  (void);
		BaseTag				*GetActiveTag     (void);

		void SetActiveObject  (BaseObject *op);
		void SetActiveTag     (BaseObject *op, BaseTag *ta);
		void SetActiveMaterial(BaseMaterial	*mp);
		void SetActiveRenderData(RenderData *rd);

		Bool CopyTo(BaseDocument *dest, LONG flags=0);
		void Flush(void);
		BaseDocument *GetClone(LONG flags=0);

		BaseObject *GetHighest(LONG type, Bool editor);

		BaseMaterial	*SearchMaterial   (const String &str);
		BaseMaterial	*SearchMaterialInc(const String &str);
		BaseObject		*SearchObject     (const String &str);
		BaseObject		*SearchObjectInc  (const String &str);

		Bool			GetChanged(void);
		void			SetChanged(void);

		Filename	GetName(void);
		Filename	GetPath(void);
		void			SetName(const Filename &fn);
		void			SetPath(const Filename &path);

		Real			GetLOD(void);
		void			SetLOD(Real lod);

		LONG			GetFps(void);
		void			SetFps(LONG fps);

		BaseTime	GetMinTime(void);
		void			SetMinTime(const BaseTime &t);

		BaseTime	GetMaxTime(void);
		void			SetMaxTime(const BaseTime &t);

		BaseTime	GetLoopMinTime(void);
		void			SetLoopMinTime(const BaseTime &t);

		BaseTime	GetLoopMaxTime(void);
		void			SetLoopMaxTime(const BaseTime &t);

		Bool			GetRenderLod(void);
		void			SetRenderLod(Bool lod);

		BaseTime	GetTime      (void);
		void			SetTime(const BaseTime &t);

		LONG			GetMode(void);
		void			SetMode(LONG m);

		LONG			GetAction(void);
		void			SetAction(LONG a);

		Bool			IsEditMode(void);

		Bool			StartUndo(void);
		Bool			EndUndo(void);
		Bool			AddUndo(LONG type, void *data);
		Bool			DoUndo(void);
		Bool			DoRedo(void);

		void			AnimateDocument(BaseThread *bt,Bool doexpressions,Bool doanim);
		void			AnimateObject(BaseObject *op, const BaseTime &time, LONG flags);

		BaseDocument *Polygonize(void);

		BaseDraw  *GetActiveBaseDraw(void);
		BaseDraw  *GetRenderBaseDraw(void);
		BaseDraw  *GetBaseDraw(LONG num);
		LONG		  GetSplinePlane(void);
};

class Hierarchy
{
	private:
		BaseDocument	*doc;
		BaseThread		*bt;
	public:
		virtual void *Alloc(void)=0;
		virtual void Free(void *data)=0;
		virtual void CopyTo(void *src, void *dst)=0;
		virtual Bool Do(void *data, BaseObject *op, const Matrix &mg, Bool controlobject)=0;

		Bool Run(BaseDocument *doc, Bool spheres, Real lod, LONG flags, void *startdata, BaseThread *bt);
};

RenderData		*AllocRenderData(void);
BaseDocument	*AllocBaseDocument(void);

#define FreeRenderData(v)   { if (v) C4DOS.Bl->Free(v); v=NULL;}
#define FreeBaseDocument(v) { if (v) C4DOS.Bl->Free(v); v=NULL;}

BaseDocument	*GetActiveDocument(void);
BaseDocument	*GetFirstDocument(void);
Bool					LoadFile(const Filename &name);

void InsertBaseDocument	(BaseDocument *doc);
void SetActiveDocument	(BaseDocument *doc);
void KillDocument				(BaseDocument *doc);

BaseDocument *LoadDocument(const Filename &name, Bool show_error);
Bool SaveDocument(BaseDocument *doc, const Filename &name, Bool show_error, LONG format);

LONG RenderDocument(BaseDocument *doc, ProgressHook *pr, void *private_data, BaseBitmap *bmp,
									  const BaseContainer &rdata, Bool external, Bool show_errors, Thread *th, LONG v_left=NOTOK, LONG v_top=NOTOK, LONG v_right=NOTOK, LONG v_bottom=NOTOK);

#endif
