#ifndef __C4DGENERAL_H
#define __C4DGENERAL_H

#include "c4d_string.h"
#include "operatingsystem.h"

class Filename;

#define UNIT_NONE	0 
#define UNIT_KM		1 
#define UNIT_M		2 
#define UNIT_CM		3 
#define UNIT_MM		4 
#define UNIT_UM		5 
#define UNIT_NM		6 
#define UNIT_MILE	7 
#define UNIT_YARD	8 
#define UNIT_FEET	9 
#define UNIT_INCH	10

#define ANGLE_DEG	0 
#define ANGLE_RAD 2

class Parser
{
	private:
		Parser(void);
		~Parser(void);
	public:
		Bool AddVar(const String &str, Real *value);
		Bool Eval(const String &str, LONG *error, Real *res, LONG unit, LONG angletype, LONG basis);
};

Parser *AllocParser(void);
void FreeParser(Parser *&pr);

struct SerialInfo
{
	String nr,name,organization,street,city,country;
};

void GeGetSerialInfo(LONG type, SerialInfo *si);
LONG GeGetVersionType(void);
void GeShowMouse(LONG v);
void GeGetSysTime(LONG *year, LONG *month, LONG *day, LONG *hour, LONG *minute, LONG *second);
inline LONG GeGetTimer(void) { return C4DOS.Ge->GetTimer(); }
String GeGetLineEnd(void);
LONG GeGetDefaultFPS(void);
LONG GeOutString(const String &str, LONG flags);
LONG GeGetCurrentOS(void);
LONG GeGetByteOrder(void);
void GeGetGray(LONG *r, LONG *g, LONG *b);
Bool GeChooseColor(Vector *col);
Bool GeRegisterPlugin(LONG type, LONG id, const String &str, void *data, LONG datasize);
void GePrint(const String &str);
Bool GeGetMovieInfo(const Filename &fn, LONG *frames, LONG *fps);
inline LONG GetC4DVersion(void) { return C4DOS.version; }

inline void lSwap (void *adr, LONG cnt=1) { C4DOS.Ge->lSwap(adr,cnt); }
inline void wSwap (void *adr, LONG cnt=1) { C4DOS.Ge->wSwap(adr,cnt); }
inline void lIntel(void *adr, LONG cnt=1) { C4DOS.Ge->lIntel(adr,cnt); }
inline void wIntel(void *adr, LONG cnt=1) { C4DOS.Ge->wIntel(adr,cnt); }
inline void lMotor(void *adr, LONG cnt=1) { C4DOS.Ge->lMotor(adr,cnt); }
inline void wMotor(void *adr, LONG cnt=1) { C4DOS.Ge->wMotor(adr,cnt); }

void SetMousePointer(LONG);
Bool ShowBitmap(const Filename &fn);
Bool ShowBitmap(BaseBitmap *bm);
void StopAllThreads(void);
void StatusClear(void);
void StatusSetSpin(void);
void StatusSetBar(LONG p);
void StatusSetText(const String &str);
void GeEventAdd(LONG messageid, ULONG p1=0, ULONG p2=0);
Bool DrawViews(LONG flags);
void *SendModelingCommand(LONG command, BaseDocument *doc, BaseObject *op, BaseContainer *bc, LONG mode); 
Filename GetGlobalTexturePath(LONG i);
void SetGlobalTexturePath(LONG i, const Filename &fn);
Bool GenerateTexturePath(const Filename &docpath, const Filename &srcname, Filename *dstname);
void FlushTexture(const Filename &docpath, const String &name);
void FlushUnusedTextures(void);
BaseContainer GetWorldContainer(void);
Vector GetWorldColor(LONG i);
void ErrorStringDialog(LONG type, Real x, Real y, LONG is);
Bool ReadPluginInfo(LONG pluginid, void *buffer, LONG size);
Bool WritePluginInfo(LONG pluginid, void *buffer, LONG size);

class Registry
{
	private:
		Registry();
		~Registry();
	public:
		LONG GetMainID(void);
		LONG GetSubID(void);
		void *GetData(void);
};

Bool GeRegistryAdd(LONG sub_id, LONG main_id, void *data);
Bool GeRegistryRemove(LONG sub_id, LONG main_id);
Registry *GeRegistryFind(LONG sub_id, LONG main_id);
Registry *GeRegistryFindLast(LONG main_id);
Registry *GeRegistryFindFirst(LONG main_id);
Registry *GeRegistryFindPrev(Registry *last);
Registry *GeRegistryFindNext(Registry *last);
Bool GeRegistryGetAutoID(LONG *id);

#endif
