#include "c4d_string.h"
#include "c4d_memory.h"

String::String(void)
{
	str=C4DOS.St->Alloc();
	if (!str) str=C4DOS.St->Default;
}

String::String(const String &cs)
{
	str=C4DOS.St->Alloc();
	if (!str) { str=C4DOS.St->Default; return; }
	Insert(0,cs);
}

String::String(_String *cs, Bool preallocated)
{
	if (preallocated || !cs)
		str = cs;
	else
	{
		str = C4DOS.St->GetClone(cs);
	}
	if (!str) str=C4DOS.St->Default; 
}

String::String(const CHAR *cstr)
{
	str=C4DOS.St->AllocCString((CHAR*)cstr,-1,StXbit);
	if (!str) str=C4DOS.St->Default;
}

String::String(LONG count,UWORD fillch)
{
	str=C4DOS.St->AllocArray(count,fillch);
	if (!str) str=C4DOS.St->Default;
}

String::~String(void)
{
	if (str==NULL || str==C4DOS.St->Default) return;
	C4DOS.St->Free(str);
	str=NULL;
}

void String::CopyTo(String *str)
{
	if (!str->Get() || !Get()) return;
	C4DOS.St->Insert(str->Get(),0,Get());
}

String *String::GetClone(void)
{
	String *str = gNew String;
	if (!str) return NULL;
	CopyTo(str);
	return str;
}

LONG String::GetLength(void) const
{
	if (!str) return 0;
	return C4DOS.St->GetLength(str);
}

const String& String::operator = (const String &cs)
{
	if (str && str!=C4DOS.St->Default) 
		C4DOS.St->Free(str);

	str=C4DOS.St->Alloc();
	if (str) Insert(0,cs); else str=C4DOS.St->Default;
	return *this;
}

const String operator + (const String &Str1,const String &Str2)
{
	return String(Str1)+=Str2;
}

const String& String::operator += (const String &Str)
{
	Insert(GetLength(),Str);
	return *this;
}

Bool String::operator < (const String &Str) const
{
	return (Compare(Str) < 0);
}

Bool String::operator > (const String &Str) const
{
	return (Compare(Str) > 0);
}

Bool String::operator <= (const String &Str) const
{
	return (Compare(Str) <= 0);
}

Bool String::operator >= (const String &Str) const
{
	return (Compare(Str) >= 0);
}

Bool String::operator == (const String &Str) const
{
	return (Compare(Str) == 0);
}

Bool String::operator != (const String &Str) const
{
	return (Compare(Str) != 0);
}

LONG String::Compare(const String &cs) const
{
	if (!str || !cs.str) return 1;
	return C4DOS.St->Compare(this->str,cs.str,0);
}

LONG String::LexCompare(const String &cs) const
{
	if (!str || !cs.str) return 1;
	return C4DOS.St->Compare(this->str,cs.str,1);
}

LONG String::RelCompare(const String &cs) const
{
	if (!str || !cs.str) return 1;
	return C4DOS.St->Compare(this->str,cs.str,2);
}

Bool String::FindFirst(const String &cs,LONG *pos,LONG start) const
{
	if (!str || !cs.str) return FALSE;
	return C4DOS.St->FindFirst(str,cs.str,pos,start);
}

Bool String::FindLast(const String &cs,LONG *pos,LONG start) const
{
	if (!str || !cs.str) return FALSE;
	return C4DOS.St->FindLast(str,cs.str,pos,start);
}

Bool String::FindFirst(UWORD ch,LONG *pos,LONG start) const
{
	Bool ok;
	if (!str) return FALSE;
	_String *tmp = C4DOS.St->AllocArray(1,ch);
	if (!tmp) return FALSE;
	ok = C4DOS.St->FindFirst(str,tmp,pos,start);
	C4DOS.St->Free(tmp);
	return ok;
}

Bool String::FindLast(UWORD ch,LONG *pos,LONG start) const
{
	Bool ok;
	if (!str) return FALSE;
	_String *tmp = C4DOS.St->AllocArray(1,ch);
	if (!tmp) return FALSE;
	ok = C4DOS.St->FindLast(str,tmp,pos,start);
	C4DOS.St->Free(tmp);
	return ok;
}

void String::Delete(LONG pos,LONG count)
{
	if (!str) return;
	C4DOS.St->Delete(str,pos,count);
}

void String::Insert(LONG pos,UWORD ch)
{
	if (!str) return;
	_String *tmp = C4DOS.St->AllocArray(1,ch);
	if (!tmp) return;
	C4DOS.St->Insert(str,pos,tmp);
	C4DOS.St->Free(tmp);
}

void String::Insert(LONG pos,const String &cs)
{
	if (!str) return;
	C4DOS.St->Insert(str,pos,cs.str);
}

const String String::SubStr(LONG start,LONG count) const
{
	if (!str) return String();
	return String(C4DOS.St->SubStr(str,start,count),TRUE);
}

Real String::StringToReal(LONG *error, LONG unit, LONG angletype, LONG base) const
{
	if (!str) return 0.0;
	return C4DOS.St->ToReal(str,error,unit,angletype,base);
}

LONG String::StringToLong(LONG *error) const
{
	if (!str) return 0;
	return C4DOS.St->ToLong(str,error);
}

const String String::ToUpper() const
{
	if (!str) return String();
	return String(C4DOS.St->ToUpper(str),TRUE);
}

const String String::ToLower() const
{
	if (!str) return String();
	return String(C4DOS.St->ToLower(str),TRUE);
}

LONG String::GetCStringLen(StringEncoding type) const
{
	if (!str) return 0;
	return C4DOS.St->GetCStringLen(str,type);
}

LONG String::GetCString(CHAR *scstr, LONG max, StringEncoding type) const
{
	if (!str) return 0;
	return C4DOS.St->GetCString(str,scstr,max,type);
}

LONG String::SetCString(const CHAR *cstr, LONG count, StringEncoding type)
{
	if (str && str!=C4DOS.St->Default) 
		C4DOS.St->Free(str);

	str=C4DOS.St->AllocCString((CHAR*)cstr,count,type);
	if (!str) { str=C4DOS.St->Default; return 0; }
	return GetLength();
}

const String LongToString(LONG l)
{
	return String(C4DOS.St->LongToString(l),TRUE);
}

const String RealToString(Real v, LONG vvk, LONG nnk, Bool e, UWORD xchar)
{
	return String(C4DOS.St->RealToString(v,vvk,nnk,e,xchar),TRUE);
}

const String::PChar String::operator[] (LONG Pos) const
{
	if (Pos >= GetLength() || Pos < 0) Pos = -1;
	return PChar((String *)(this),Pos);
}

String::PChar String::operator[] (LONG Pos)
{
	if (Pos >= GetLength() || Pos < 0) Pos = -1;
	return PChar(this,Pos);
}

String::PChar::PChar(String *str,LONG pos)
{
	Str = str;
	Pos = pos;
}

String::PChar& String::PChar::operator = (const PChar& rhs)
{
	if (!Str) return *this;
	if (Pos < 0 || Pos >= Str->GetLength()) return *this;
	
	if (!Str->str || !rhs.Str->str) return *this;
	C4DOS.St->SetChr(Str->str,Pos,C4DOS.St->GetChr(rhs.Str->str,rhs.Pos));
	return *this;
}

String::PChar& String::PChar::operator = (UWORD c)
{
	if (!Str) return *this;
	if (Pos < 0 || Pos >= Str->GetLength()) return *this;
	C4DOS.St->SetChr(Str->str,Pos,c);
	return *this;
}

String::PChar::operator UWORD(void) const
{
	if (!Str) return 0;
	if (Pos < 0 || Pos >= Str->GetLength()) return 0;
	else return C4DOS.St->GetChr(Str->str,Pos);
}

#if (defined _DEBUG) && (defined WIN32)
static CHAR array[1000];

CHAR *cstr(String &str)
{
	str.GetCString(array,1000);
	return array;
}

#include "c4d_file.h"
CHAR *cstr(Filename &fn)
{
	fn.GetString().GetCString(array,1000);
	return array;
}
#endif