#ifndef __SYSMATH_H
#define __SYSMATH_H

#include <math.h>

#pragma warning(disable:4244)
#pragma warning(disable:4514)
#pragma warning(disable:4710)
#pragma warning(disable:4702)
#pragma warning(disable:4305)
#pragma warning(disable:4291)

#define __TYPES_READY

struct Matrix;
struct Vector;

#define FALSE 0
#define TRUE  1
#define NULL 	0


#ifndef __WINDOWS_H
#ifndef _WINDOWS_
typedef short int	          WORD;
#endif
#endif
typedef unsigned short int	UWORD;
typedef long 								LONG;
typedef unsigned long				ULONG;
typedef int                 Bool;
typedef float								SReal;
typedef double							LReal;
typedef void*								Hndl;
typedef int									INT;
typedef unsigned int 				UINT;
typedef char								CHAR;
typedef unsigned char				UCHAR;

#ifdef __COPRO
typedef double							Real;
#define MAXREAL							1.0E+308
#else
typedef float								Real;
#define MAXREAL							9.0e18 // Dieser Wert muss auf PC/MAC korrigiert werden!
#endif

#define pi					3.1415926535897932384626433832795
#define pi2					6.283185307179586476925286766559
#define pi05				1.5707963267948966192313216916398
#define sqrt2				1.4142135623730950488016887242097
#define onesqrt2		0.70710678118654752440084436210485

#ifndef MAXLONG
#define MAXLONG							2147483647
#endif
#define MAXULONG						4294967295
#ifndef MAXWORD
#define MAXWORD							32767
#endif
#define MAXUWORD						65535

inline LONG SAFELONG(Real x)
{
	if(x<=-MAXLONG) return -MAXLONG;
	else if(x>=MAXLONG) return MAXLONG;
	return LONG(x);
}

inline Real Rad(Real r){return ((r)*pi/180.0);}
inline Real Deg(Real r){return ((r)*180.0/pi);}

#define MMin(a,b)		(((a)<(b))?(a):(b))
#define MMax(a,b)		(((a)>(b))?(a):(b))
#define Cut(a,b,c)	MMax(MMin((a),(c)),(b))

#define Abs(a)		(((a)<0)?-(a):(a))
#define Mod(a,b)	((a)<0?((a)-(((a)-(b)+1)/(b))*(b)):(a)%(b))
#define Sin(r) 		(Real(sin 	(double(r))))
#define Cos(r) 		(Real(cos 	(double(r))))
#define Tan(r) 		(Real(tan 	(double(r))))
#define ATan(r) 	(Real(atan	(double(r))))
#define Exp(r)		(Real(exp 	(double(r))))
#define Ln(r) 		(Real(log 	(double(r))))
#define Ln10(r) 	(Real(log10	(double(r))))
#define Sqrt(r)		(Real(sqrt	(double(r))))
#define Floor(r)	(Real(floor	(double(r))))
#define Ceil(r)		(Real(ceil	(double(r))))
#define Pow(r,s)	(Real(pow		(double(r),double(s))))
#define Cosh(r)		(Real(cosh	(double(r))))
#define Sinh(r)		(Real(sinh	(double(r))))
#define Tanh(r)		(Real(tanh	(double(r))))
#define FMod(r,s)	(Real(fmod	(double(r),double(s))))

inline void SinCos(Real w,Real &sn,Real &cn)
{
	sn=Sin(w);
	cn=Cos(w);
}//SinCos

inline Real ASin(Real val)
{
	if(val>=1.0) return pi05;
	else if(val<=-1.0) return -pi05;
	return asin((double)val);
}

inline Real ACos(Real val)
{
	if(val>=1.0) return 0.0;
	else if(val<=-1.0) return pi;
	return acos((double)val);
}

#endif
