#include "messiahHOST_.h"

#include "c4d_memory.h"
#include "c4d_baseobject.h"

#include "messiahManager.h"

#include "ConfigData.h"

__EXPORTVAR(int) hostObjectGeometryCopy_(FX_NodeLog *messiah_node, int add_to_hierarchy)
{
	
  if (!messiah_node || !messiah_node->element || !messiah_node->obj) return FALSE;

  ConfigData *config = GetConfigData();
  if(!config) return FALSE;

  // Get the data we need from the manager
  BaseObject* obj = mm->getGeometryFromElement(messiah_node->element);
  if (!obj) return FALSE;

  // Everything below is a recursion over an object hierarchy,
  // flattening it as it goes along. Perhaps there's a better way.

  // Record the points and polygons (I hope this is OK)
  BaseObject* child = obj; // The first time it's not a "child"
  LONG processedPoints = 0;

  //Record a dummy surface for now
  nodeObjectCreateSurface(messiah_node->obj, "Default");
 
  while (child)
  {
    LONG pointsAdded = 0;

    // Only process point objects
    if (child->GetLayerType() & L_pointobject)
    {
      Matrix mg = child->GetMg();

      PointObject* pobj = static_cast<PointObject*>(child);
      LONG pointCount = pobj->GetPointCount();
      Vector* points = pobj->GetPoint();
      for (int i = 0; i < pointCount; ++i)
      {
        Vector localPoint = mg * points[i];
        nodeObjectCreatePoint(messiah_node->obj, 
          localPoint.x * config->scaleFactor, localPoint.y * config->scaleFactor, localPoint.z * config->scaleFactor);
      }
      pointsAdded += pointCount;
    }

    // Only process polygon objects
    if (child->GetType() == Opolygon)
    {
      PolygonObject* pobj = static_cast<PolygonObject*>(child);
      LONG polyCount = pobj->GetPolygonCount();
      Polygon* polys = pobj->GetPolygon();
      for (int i = 0; i < polyCount; ++i)
      {
        LONG numPoints = (polys[i].c == polys[i].d) ? 3 : 4;

        // I this gives the right memory structure
        int accPoints[4];
        accPoints[0] = polys[i].a + processedPoints;
        accPoints[1] = polys[i].b + processedPoints;
        accPoints[2] = polys[i].c + processedPoints;
        accPoints[3] = polys[i].d + processedPoints;

        nodeObjectCreatePoly(messiah_node->obj, 
		    	numPoints, &accPoints[0], 1, 0);
      }
    }

    processedPoints += pointsAdded;
    child = GetNextHierarchyObject(child, obj);
  }


  // End the node
	nodeObjectEnd(messiah_node->obj, add_to_hierarchy);

  return TRUE;
}


/*
__EXPORTVAR(int) hostObjectGeometryCopy_(FX_NodeLog *messiah_node, int add_to_hierarchy)
{
	
	GePrint("hostObjectGeometryCopy");

	if (!messiah_node || !messiah_node->element || !messiah_node->obj) return FALSE;

	// Get the data we need from the manager
	BaseObject* obj = mm->getGeometryFromElement(messiah_node->element);
	if ( !obj ) return FALSE;
    if ( !(obj->GetType() == Opolygon) ) return FALSE;

	// Only process point objects
	if (obj->GetLayerType() & L_pointobject)
	{

		PointObject* pobj = static_cast<PointObject*>(obj);
		LONG pointCount = pobj->GetPointCount();
		Vector* points = pobj->GetPoint();
		for (int i = 0; i < pointCount; ++i)
		{
			Vector localPoint = points[i]; 
			GePrint("Create Point");
			nodeObjectCreatePoint(messiah_node->obj, 
			  localPoint.x, localPoint.y, localPoint.z);
		}
	}

	//Record a dummy surface for now
	GePrint("Create Surface");
	nodeObjectCreateSurface(messiah_node->obj, "Dummy");


    // Only process polygon objects
    if (obj->GetType() == Opolygon)
    {
      PolygonObject* pobj = static_cast<PolygonObject*>(obj);
      LONG polyCount = pobj->GetPolygonCount();
      Polygon* polys = pobj->GetPolygon();
      for (int i = 0; i < polyCount; ++i)
      {
        LONG numPoints = (polys[i].c == polys[i].d) ? 3 : 4;

        // I this gives the right memory structure
        int accPoints[4];
        accPoints[0] = polys[i].a; 
        accPoints[1] = polys[i].b; 
        accPoints[2] = polys[i].c; 
        accPoints[3] = polys[i].d; 

		GePrint("Create Poly");
        nodeObjectCreatePoly(messiah_node->obj, 
		    	numPoints, &accPoints[0], 1, 0);
      }
    }

	// End the node
	GePrint("EndObject");

	nodeObjectEnd(messiah_node->obj, add_to_hierarchy);

	GePrint("End Geometry Copy");

	return TRUE;

}
*/